@@ -274,7 +274,7 @@ class SentenceTransformer(nn.Sequential):
                 chunk = []
 
         if len(chunk) > 0:
-            input_queue.put([last_chunk_id, chunk])
+            input_queue.put([last_chunk_id, is_pretokenized, chunk])
             last_chunk_id += 1
 
         output_queue = pool['output']
@@ -609,9 +609,10 @@ class SentenceTransformer(nn.Sequential):
             score = evaluator(self, output_path=output_path, epoch=epoch, steps=steps)
             if callback is not None:
                 callback(score, epoch, steps)
-            if score > self.best_score and save_best_model:
-                self.save(output_path)
+            if score > self.best_score:
                 self.best_score = score
+                if save_best_model:
+                    self.save(output_path)
 
 
     def _get_scheduler(self, optimizer, scheduler: str, warmup_steps: int, t_total: int):
